@@ -264,8 +264,10 @@ DEFAULT_LANGUAGE = 'en'
 # This needs to be updated every time a new language is added to spyder, and is
 # used by the Preferences configuration to populate the Language QComboBox
 LANGUAGE_CODES = {'en': u'English',
+                  'de': u'Deutsch',
                   'fr': u'Français',
                   'es': u'Español',
+                  'hu': u'Magyar',
                   'pt_BR': u'Português',
                   'ru': u'Русский',
                   'ja': u'日本語'
@@ -294,9 +296,10 @@ def get_available_translations():
     # is added, to ensure LANGUAGE_CODES is updated.
     for lang in langs:
         if lang not in LANGUAGE_CODES:
-            error = _('Update LANGUAGE_CODES (inside config/base.py) if a new '
-                      'translation has been added to Spyder')
-            raise Exception(error)
+            error = ('Update LANGUAGE_CODES (inside config/base.py) if a new '
+                     'translation has been added to Spyder')
+            print(error)  # spyder: test-skip
+            return ['en']
     return langs
 
 
@@ -307,11 +310,11 @@ def get_interface_language():
     otherwise it will return DEFAULT_LANGUAGE.
 
     Example:
-    1.) Spyder provides ('en',  'fr', 'es' and 'pt_BR'), if the locale is
-    either 'en_US' or 'en' or 'en_UK', this function will return 'en'
+    1.) Spyder provides ('en',  'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
+    locale is either 'en_US' or 'en' or 'en_UK', this function will return 'en'
 
-    2.) Spyder provides ('en',  'fr', 'es' and 'pt_BR'), if the locale is
-    either 'pt' or 'pt_BR', this function will return 'pt_BR'
+    2.) Spyder provides ('en',  'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
+    locale is either 'pt' or 'pt_BR', this function will return 'pt_BR'
     """
 
     # Solves issue #3627
