@@ -34,7 +34,7 @@ from spyder.utils import icon_manager as ima
 from spyder.utils import sourcecode
 from spyder.utils.encoding import get_coding
 from spyder.utils.environ import RemoteEnvDialog
-from spyder.utils.programs import TEMPDIR
+from spyder.utils.programs import get_temp_dir
 from spyder.utils.qthelpers import (add_actions, create_action,
                                     create_toolbutton, DialogManager,
                                     MENU_SEPARATOR)
@@ -102,7 +102,8 @@ class ClientWidget(QWidget, SaveHistoryMixin):
                  menu_actions=None, slave=False,
                  external_kernel=False, given_name=None,
                  show_elapsed_time=False,
-                 reset_warning=True):
+                 reset_warning=True,
+                 ask_before_restart=True):
         super(ClientWidget, self).__init__(plugin)
         SaveHistoryMixin.__init__(self, history_filename)
 
@@ -116,6 +117,7 @@ class ClientWidget(QWidget, SaveHistoryMixin):
         self.given_name = given_name
         self.show_elapsed_time = show_elapsed_time
         self.reset_warning = reset_warning
+        self.ask_before_restart = ask_before_restart
 
         # --- Other attrs
         self.options_button = None
@@ -125,6 +127,7 @@ class ClientWidget(QWidget, SaveHistoryMixin):
         self.history = []
         self.allow_rename = True
         self.stderr_dir = None
+        self.is_error_shown = False
 
         # --- Widgets
         self.shellwidget = ShellWidget(config=config_options,
@@ -186,13 +189,37 @@ class ClientWidget(QWidget, SaveHistoryMixin):
                 stderr_file = osp.join(self.stderr_dir, stderr_file)
             else:
                 try:
-                    if not osp.isdir(TEMPDIR):
-                        os.makedirs(TEMPDIR)
-                    stderr_file = osp.join(TEMPDIR, stderr_file)
+                    stderr_file = osp.join(get_temp_dir(), stderr_file)
                 except (IOError, OSError):
                     stderr_file = None
         return stderr_file
 
+    @property
+    def stderr_handle(self):
+        """Get handle to stderr_file."""
+        if self.stderr_file is not None:
+            # Needed to prevent any error that could appear.
+            # See issue 6267
+            try:
+                handle = codecs.open(self.stderr_file, 'w', encoding='utf-8')
+            except Exception:
+                handle = None
+        else:
+            handle = None
+
+        return handle
+
+    def remove_stderr_file(self):
+        """Remove stderr_file associated with the client."""
+        try:
+            # Defer closing the stderr_handle until the client
+            # is closed because jupyter_client needs it open
+            # while it tries to restart the kernel
+            self.stderr_handle.close()
+            os.remove(self.stderr_file)
+        except Exception:
+            pass
+
     def configure_shellwidget(self, give_focus=True):
         """Configure shellwidget after kernel is started"""
         if give_focus:
@@ -274,7 +301,7 @@ class ClientWidget(QWidget, SaveHistoryMixin):
             error = error.replace(eol, '<br>')
 
         # Don't break lines in hyphens
-        # From http://stackoverflow.com/q/7691569/438386
+        # From https://stackoverflow.com/q/7691569/438386
         error = error.replace('-', '&#8209')
 
         # Create error page
@@ -285,10 +312,13 @@ class ClientWidget(QWidget, SaveHistoryMixin):
                                                 error=error)
 
         # Show error
-        self.infowidget.setHtml(page)
+        self.infowidget.setHtml(page, QUrl.fromLocalFile(CSS_PATH))
         self.shellwidget.hide()
         self.infowidget.show()
 
+        # Tell the client we're in error mode
+        self.is_error_shown = True
+
     def get_name(self):
         """Return client name"""
         if self.given_name is None:
@@ -473,7 +503,7 @@ class ClientWidget(QWidget, SaveHistoryMixin):
         """
         sw = self.shellwidget
 
-        if not running_under_pytest():
+        if not running_under_pytest() and self.ask_before_restart:
             message = _('Are you sure you want to restart the kernel?')
             buttons = QMessageBox.Yes | QMessageBox.No
             result = QMessageBox.question(self, _('Restart kernel?'),
@@ -481,13 +511,16 @@ class ClientWidget(QWidget, SaveHistoryMixin):
         else:
             result = None
 
-        if result == QMessageBox.Yes or running_under_pytest():
+        if (result == QMessageBox.Yes or
+                running_under_pytest() or
+                not self.ask_before_restart):
             if sw.kernel_manager:
                 if self.infowidget.isVisible():
                     self.infowidget.hide()
                     sw.show()
                 try:
-                    sw.kernel_manager.restart_kernel()
+                    sw.kernel_manager.restart_kernel(
+                        stderr=self.stderr_handle)
                 except RuntimeError as e:
                     sw._append_plain_text(
                         _('Error restarting kernel: %s\n') % e,
@@ -509,27 +542,23 @@ class ClientWidget(QWidget, SaveHistoryMixin):
     @Slot(str)
     def kernel_restarted_message(self, msg):
         """Show kernel restarted/died messages."""
-        try:
-            stderr = codecs.open(self.stderr_file, 'r',
-                                 encoding='utf-8').read()
-        except UnicodeDecodeError:
-            # This is needed since the stderr file could be encoded
-            # in something different to utf-8.
-            # See issue 4191
+        if not self.is_error_shown:
+            # If there are kernel creation errors, jupyter_client will
+            # try to restart the kernel and qtconsole prints a
+            # message about it.
+            # So we read the kernel's stderr_file and display its
+            # contents in the client instead of the usual message shown
+            # by qtconsole.
             try:
                 stderr = self._read_stderr()
-            except:
+            except Exception:
                 stderr = None
-        except (OSError, IOError):
-            stderr = None
-
-        if stderr:
-            self.show_kernel_error('<tt>%s</tt>' % stderr)
+            if stderr:
+                self.show_kernel_error('<tt>%s</tt>' % stderr)
         else:
             self.shellwidget._append_html("<br>%s<hr><br>" % msg,
                                           before_prompt=False)
 
-
     @Slot()
     def inspect_object(self):
         """Show how to inspect an object with our Help plugin"""
@@ -639,10 +668,27 @@ class ClientWidget(QWidget, SaveHistoryMixin):
 
     def _read_stderr(self):
         """Read the stderr file of the kernel."""
-        stderr_text = open(self.stderr_file, 'rb').read()
-        encoding = get_coding(stderr_text)
-        stderr = to_text_string(stderr_text, encoding)
-        return stderr
+        # We need to read stderr_file as bytes to be able to
+        # detect its encoding with chardet
+        f = open(self.stderr_file, 'rb')
+
+        try:
+            stderr_text = f.read()
+
+            # This is needed to avoid showing an empty error message
+            # when the kernel takes too much time to start.
+            # See issue 8581
+            if not stderr_text:
+                return ''
+
+            # This is needed since the stderr file could be encoded
+            # in something different to utf-8.
+            # See issue 4191
+            encoding = get_coding(stderr_text)
+            stderr_text = to_text_string(stderr_text, encoding)
+            return stderr_text
+        finally:
+            f.close()
 
     def _show_mpl_backend_errors(self):
         """
